[lld] r335583 - [ELF] - Change the way of sorting local symbols.
George Rimar via llvm-commits
llvm-commits at lists.llvm.org
Tue Jun 26 01:50:09 PDT 2018
Author: grimar
Date: Tue Jun 26 01:50:09 2018
New Revision: 335583
URL: http://llvm.org/viewvc/llvm-project?rev=335583&view=rev
Log:
[ELF] - Change the way of sorting local symbols.
rLLD329787 added the stable sorting to SymbolTableBaseSection::postThunkContents.
I profiled the Mozilla (response-O0.txt) from lld-speed-test package and found
std::stable_sort is showing up in profile results and consuming the 3.1% of the total
CPU time in the RelWithDebug build. Total time of postThunkContents is 3.54%, 238ms.
This change reduces postTimeContents time to 50ms, making it to take 0.73% of Total CPU time.
So, instead of sorting the local part I suggest to just rebuild it.
That is what this patch does.
Differential revision: https://reviews.llvm.org/D45519
Modified:
lld/trunk/ELF/SyntheticSections.cpp
Modified: lld/trunk/ELF/SyntheticSections.cpp
URL: http://llvm.org/viewvc/llvm-project/lld/trunk/ELF/SyntheticSections.cpp?rev=335583&r1=335582&r2=335583&view=diff
==============================================================================
--- lld/trunk/ELF/SyntheticSections.cpp (original)
+++ lld/trunk/ELF/SyntheticSections.cpp Tue Jun 26 01:50:09 2018
@@ -1762,19 +1762,19 @@ void SymbolTableBaseSection::postThunkCo
size_t NumLocals = E - Symbols.begin();
getParent()->Info = NumLocals + 1;
- // Assign the growing unique ID for each local symbol's file.
- DenseMap<InputFile *, unsigned> FileIDs;
- for (auto I = Symbols.begin(); I != E; ++I)
- FileIDs.insert({I->Sym->File, FileIDs.size()});
+ // We want to group the local symbols by file. For that we rebuild the local
+ // part of the symbols vector. We do not need to care about the STT_FILE
+ // symbols, they are already naturally placed first in each group. That
+ // happens because STT_FILE is always the first symbol in the object and hence
+ // precede all other local symbols we add for a file.
+ MapVector<InputFile *, std::vector<SymbolTableEntry>> Arr;
+ for (const SymbolTableEntry &S : llvm::make_range(Symbols.begin(), E))
+ Arr[S.Sym->File].push_back(S);
- // Sort the local symbols to group them by file. We do not need to care about
- // the STT_FILE symbols, they are already naturally placed first in each group.
- // That happens because STT_FILE is always the first symbol in the object and
- // hence precede all other local symbols we add for a file.
- std::stable_sort(Symbols.begin(), E,
- [&](const SymbolTableEntry &L, const SymbolTableEntry &R) {
- return FileIDs[L.Sym->File] < FileIDs[R.Sym->File];
- });
+ auto I = Symbols.begin();
+ for (std::pair<InputFile *, std::vector<SymbolTableEntry>> &P : Arr)
+ for (SymbolTableEntry &Entry : P.second)
+ *I++ = Entry;
}
void SymbolTableBaseSection::addSymbol(Symbol *B) {
More information about the llvm-commits
mailing list